@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2023 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,6 +11,8 @@
 
 import os
 import signal
+from typing import Dict, Any, Optional, Callable
+from gi.repository import GLib  # @UnresolvedImport
 
 from xpra.util import envint, envbool
 from xpra.os_util import POSIX
@@ -18,35 +20,26 @@ from xpra.log import Logger
 
 log = Logger("server", "util", "exec")
 
-USE_PROCESS_POLLING = not POSIX or envbool("XPRA_USE_PROCESS_POLLING")
-POLL_DELAY = envint("XPRA_POLL_DELAY", 2)
 
-
-singleton = None
-def getChildReaper():
-    global singleton
-    if singleton is None:
-        singleton = ChildReaper()
-    return singleton
-
-
-def reaper_cleanup():
-    global singleton
-    s = singleton
-    if not s:
-        return
-    singleton.cleanup()
-    #keep it around,
-    #so we don't try to reinitialize it from the wrong thread
-    #(signal requires the main thread)
-    #singleton = None
-
-
-class ProcInfo(object):
+class ProcInfo:
+    __slots__ = (
+        "pid", "name", "command", "ignore",
+        "forget", "dead", "returncode",
+         "callback", "process",
+         )
+    pid : int
+    name : str
+    command : Any
+    ignore : bool
+    forget : bool
+    dead : bool
+    returncode : Optional[int]
+    callback : Optional[Callable]
+    process : Any
     def __repr__(self):
-        return "ProcInfo(%s)" % self.__dict__
+        return f"ProcInfo({self.pid} : {self.command})"
 
-    def get_info(self):
+    def get_info(self) -> Dict[str,Any]:
         info = {
             "pid"       : self.pid,
             "name"      : self.name,
@@ -70,17 +63,18 @@ class ProcInfo(object):
 #
 # WNOHANG is a tricky beast, see:
 # https://github.com/gevent/gevent/issues/622
-class ChildReaper(object):
+class ChildReaper:
+    __slots__ = ("_quit", "_proc_info")
     #note: the quit callback will fire only once!
     def __init__(self, quit_cb=None):
         log("ChildReaper(%s)", quit_cb)
-        from xpra.gtk_common.gobject_compat import import_glib
-        self.glib = import_glib()
         self._quit = quit_cb
         self._proc_info = []
+        USE_PROCESS_POLLING = not POSIX or envbool("XPRA_USE_PROCESS_POLLING")
         if USE_PROCESS_POLLING:
+            POLL_DELAY = envint("XPRA_POLL_DELAY", 2)
             log("using process polling every %s seconds", POLL_DELAY)
-            self.glib.timeout_add(POLL_DELAY*1000, self.poll)
+            GLib.timeout_add(POLL_DELAY*1000, self.check)
         else:
             signal.signal(signal.SIGCHLD, self.sigchld)
             # Check once after the mainloop is running, just in case the exit
@@ -89,15 +83,15 @@ class ChildReaper(object):
             def check_once():
                 self.check()
                 return False # Only call once
-            self.glib.timeout_add(0, check_once)
+            GLib.timeout_add(0, check_once)
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         self.reap()
         self.poll()
         self._proc_info = []
         self._quit = None
 
-    def add_process(self, process, name, command, ignore=False, forget=False, callback=None):
+    def add_process(self, process, name : str, command, ignore=False, forget=False, callback=None) -> ProcInfo:
         pid = process.pid
         assert pid>0, "process has no pid!"
         procinfo = ProcInfo()
@@ -110,14 +104,14 @@ class ChildReaper(object):
         procinfo.process = process
         procinfo.returncode = process.poll()
         procinfo.dead = procinfo.returncode is not None
-        log("add_process(%s, %s, %s, %s, %s) pid=%s", process, name, command, ignore, forget, pid)
+        log("add_process%s pid=%s", (process, name, command, ignore, forget, callback), pid)
         #could have died already:
         self._proc_info.append(procinfo)
         if procinfo.dead:
             self.add_dead_process(procinfo)
         return procinfo
 
-    def poll(self):
+    def poll(self) -> bool:
         #poll each process that is not dead yet:
         log("poll() procinfo list: %s", self._proc_info)
         for procinfo in tuple(self._proc_info):
@@ -126,10 +120,10 @@ class ChildReaper(object):
                 self.add_dead_process(procinfo)
         return True
 
-    def set_quit_callback(self, cb):
+    def set_quit_callback(self, cb:Callable) -> None:
         self._quit = cb
 
-    def check(self):
+    def check(self) -> bool:
         #see if we are meant to exit-with-children
         #see if we still have procinfos alive (and not meant to be ignored)
         self.poll()
@@ -146,22 +140,22 @@ class ChildReaper(object):
             return False
         return True
 
-    def sigchld(self, signum, frame):
+    def sigchld(self, signum, frame) -> None:
         #we risk race conditions if doing anything in the signal handler,
         #better run in the main thread asap:
-        self.glib.idle_add(self._sigchld, signum, str(frame))
+        GLib.idle_add(self._sigchld, signum, str(frame))
 
-    def _sigchld(self, signum, frame_str):
+    def _sigchld(self, signum, frame_str) -> None:
         log("sigchld(%s, %s)", signum, frame_str)
         self.reap()
 
-    def get_proc_info(self, pid):
+    def get_proc_info(self, pid : int) -> Optional[ProcInfo]:
         for proc_info in tuple(self._proc_info):
             if proc_info.pid==pid:
                 return proc_info
         return None
 
-    def add_dead_pid(self, pid):
+    def add_dead_pid(self, pid : int) -> None:
         #find the procinfo for this pid:
         matches = [procinfo for procinfo in self._proc_info if procinfo.pid==pid and not procinfo.dead]
         log("add_dead_pid(%s) matches=%s", pid, matches)
@@ -171,7 +165,7 @@ class ChildReaper(object):
         for procinfo in matches:
             self.add_dead_process(procinfo)
 
-    def add_dead_process(self, procinfo):
+    def add_dead_process(self, procinfo:ProcInfo) -> None:
         log("add_dead_process(%s)", procinfo)
         process = procinfo.process
         if procinfo.dead or not process:
@@ -185,7 +179,7 @@ class ChildReaper(object):
             return
         if process and cb:
             procinfo.callback = None
-            self.glib.idle_add(cb, process)
+            GLib.idle_add(cb, process)
         #once it's dead, clear the reference to the process:
         #this should free up some resources
         #and also help to ensure we don't end up here again
@@ -198,12 +192,12 @@ class ChildReaper(object):
             #forget it:
             try:
                 self._proc_info.remove(procinfo)
-            except ValueError:
+            except ValueError:  # pragma: no cover
                 log("failed to remove %s from proc info list", procinfo, exc_info=True)
         log("updated procinfo=%s", procinfo)
         self.check()
 
-    def reap(self):
+    def reap(self) -> None:
         self.poll()
         while POSIX:
             log("reap() calling os.waitpid%s", (-1, "WNOHANG"))
@@ -216,9 +210,9 @@ class ChildReaper(object):
                 break
             self.add_dead_pid(pid)
 
-    def get_info(self):
+    def get_info(self) -> Dict[Any,Any]:
         iv = tuple(self._proc_info)
-        info = {
+        info : Dict[Any,Any] = {
                 "children"  : {
                                "total"      : len(iv),
                                "dead"       : len(tuple(True for x in iv if x.dead)),
@@ -226,7 +220,7 @@ class ChildReaper(object):
                                }
                 }
         pi = sorted(self._proc_info, key=lambda x: x.pid, reverse=True)
-        cinfo = {}
+        cinfo : Dict[int,Any] = info.setdefault("child", {})
         for i, procinfo in enumerate(pi):
             d = {}
             for k in ("name", "command", "ignore", "forget", "returncode", "dead", "pid"):
@@ -235,5 +229,20 @@ class ChildReaper(object):
                     continue
                 d[k] = v
             cinfo[i] = d
-        info["child"] = cinfo
         return info
+
+singleton : Optional[ChildReaper] = None
+def getChildReaper() -> ChildReaper:
+    global singleton
+    if singleton is None:
+        singleton = ChildReaper()
+    return singleton
+
+def reaper_cleanup() -> None:
+    s = singleton
+    if s is not None:
+        s.cleanup()
+    #keep it around,
+    #so we don't try to reinitialize it from the wrong thread
+    #(signal requires the main thread)
+    #singleton = None
