@@ -14,7 +14,7 @@ from typing import Dict, Tuple, Any, Iterable
 from xpra.common import DEFAULT_XDG_DATA_DIRS
 from xpra.net.http.directory_listing import list_directory
 from xpra.net.bytestreams import pretty_socket
-from xpra.util import envbool, std, csv, AdHocStruct, repr_ellipsized
+from xpra.util import envbool, std, csv, AdHocStruct, repr_ellipsized, obsc
 from xpra.platform.paths import get_desktop_background_paths
 from xpra.log import Logger
 
@@ -291,7 +291,7 @@ class HTTPRequestHandler(BaseHTTPRequestHandler):
     def handle_authentication(self) -> bool:
         if not self.password:
             return True
-        authlog = Logger("auth", "http")
+        authlog = Logger("auth")
         def auth_err(msg):
             self.do_AUTHHEAD()
             self.wfile.write(msg.encode("latin1"))
@@ -321,7 +321,7 @@ class HTTPRequestHandler(BaseHTTPRequestHandler):
         username, password = s.split(":", 1)
         if (self.username and username!=self.username) or password!=self.password:
             authlog("http authentication: expected %s:%s but received %s:%s",
-                self.username or "", self.password, username, password)
+                self.username or "", obsc(self.password), username, obsc(password))
             return auth_err("invalid credentials")
         authlog("http authentication passed")
         return True
