@@ -1,14 +1,15 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # This file is part of Xpra.
-# Copyright (C) 2009-2016 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2009-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import avahi
+import avahi  # @UnresolvedImport
 import dbus
+from typing import Any
 
-from xpra.net.mdns import XPRA_MDNS_TYPE
+from xpra.net.mdns import XPRA_TCP_MDNS_TYPE, XPRA_UDP_MDNS_TYPE
 from xpra.dbus.common import init_system_bus
 from xpra.dbus.helper import dbus_to_native
 from xpra.log import Logger
@@ -18,7 +19,7 @@ log = Logger("network", "mdns")
 
 class AvahiListener:
 
-    def __init__(self, service_type, mdns_found=None, mdns_add=None, mdns_remove=None, mdns_update=None):
+    def __init__(self, service_type=XPRA_TCP_MDNS_TYPE, mdns_found=None, mdns_add=None, mdns_remove=None, mdns_update=None):
         log("AvahiListener%s", (service_type, mdns_found, mdns_add, mdns_remove, mdns_update))
         try:
             self.bus = init_system_bus()
@@ -36,14 +37,17 @@ class AvahiListener:
         self.mdns_add = mdns_add
         self.mdns_remove = mdns_remove
         #self.mdns_update = mdns_update
-        self.server = None
+        self.server : Any = None
+        self.sbrowser : Any = None
 
-    def resolve_error(self, *args):
+    @staticmethod
+    def resolve_error(*args) -> None:
         log.error("AvahiListener.resolve_error%s", args)
 
-    def service_resolved(self, interface, protocol, name, stype, domain, host, x, address, port, text_array, v):
+    def service_resolved(self, interface, protocol, name:str, stype:str,
+                         domain:str, host:str, x, address, port:int, text_array, v) -> None:
         log("AvahiListener.service_resolved%s",
-		(interface, protocol, name, stype, domain, host, x, address, port, "..", v))
+        (interface, protocol, name, stype, domain, host, x, address, port, "..", v))
         if self.mdns_add:
             #parse text data:
             text = {}
@@ -61,7 +65,7 @@ class AvahiListener:
             nargs = (dbus_to_native(x) for x in (interface, protocol, name, stype, domain, host, address, port, text))
             self.mdns_add(*nargs)
 
-    def service_found(self, interface, protocol, name, stype, domain, flags):
+    def service_found(self, interface, protocol, name:str, stype:str, domain:str, flags:int) -> None:
         log("service_found%s", (interface, protocol, name, stype, domain, flags))
         if flags & avahi.LOOKUP_RESULT_LOCAL:
             # local service, skip
@@ -72,20 +76,20 @@ class AvahiListener:
                 domain, avahi.PROTO_UNSPEC, dbus.UInt32(0),
                 reply_handler=self.service_resolved, error_handler=self.resolve_error)
 
-    def service_removed(self, interface, protocol, name, stype, domain, flags):
+    def service_removed(self, interface, protocol, name:str, stype:str, domain, flags:int) -> None:
         log("service_removed%s", (interface, protocol, name, stype, domain, flags))
         if self.mdns_remove:
             nargs = (dbus_to_native(x) for x in (interface, protocol, name, stype, domain, flags))
             self.mdns_remove(*nargs)
 
 
-    def start(self):
+    def start(self) -> None:
         self.server = dbus.Interface(self.bus.get_object(avahi.DBUS_NAME, '/'), 'org.freedesktop.Avahi.Server')
         log("AvahiListener.start() server=%s", self.server)
 
         self.sbrowser = dbus.Interface(self.bus.get_object(avahi.DBUS_NAME,
                                 self.server.ServiceBrowserNew(avahi.IF_UNSPEC,
-                                avahi.PROTO_UNSPEC, XPRA_MDNS_TYPE, 'local', dbus.UInt32(0))),
+                                avahi.PROTO_UNSPEC, self.service_type, 'local', dbus.UInt32(0))),
                                 avahi.DBUS_INTERFACE_SERVICE_BROWSER)
         log("AvahiListener.start() service browser=%s", self.sbrowser)
         s = self.sbrowser.connect_to_signal("ItemNew", self.service_found)
@@ -93,7 +97,7 @@ class AvahiListener:
         s = self.sbrowser.connect_to_signal("ItemRemove", self.service_removed)
         self.signal_match.append(s)
 
-    def stop(self):
+    def stop(self) -> None:
         sm = self.signal_match
         self.signal_match = []
         for s in sm:
@@ -113,14 +117,19 @@ def main():
 
     from xpra.dbus.common import loop_init
     loop_init()
-    listener = AvahiListener(XPRA_MDNS_TYPE, mdns_found, mdns_add, mdns_remove)
+    from gi.repository import GLib  # @UnresolvedImport
+    listeners = []
+    def add(service_type):
+        listener = AvahiListener(service_type, mdns_found, mdns_add, mdns_remove)
+        listeners.append(listener)
+        GLib.idle_add(listener.start)
+    add(XPRA_TCP_MDNS_TYPE)
+    add(XPRA_UDP_MDNS_TYPE)
     try:
-        from xpra.gtk_common.gobject_compat import import_glib
-        glib = import_glib()
-        glib.idle_add(listener.start)
-        glib.MainLoop().run()
+        GLib.MainLoop().run()
     finally:
-        listener.stop()
+        for l in listeners:
+            l.stop()
 
 
 if __name__ == "__main__":
