@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2014 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2014-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from typing import Tuple, Dict
+
 from xpra.server.keyboard_config_base import KeyboardConfigBase
 from xpra.log import Logger
 
@@ -12,9 +14,6 @@ log = Logger("keyboard", "osx")
 
 class KeyboardConfig(KeyboardConfigBase):
 
-    def __init__(self):
-        KeyboardConfigBase.__init__(self)
-
     def __repr__(self):
         return "osx.KeyboardConfig"
 
@@ -23,18 +22,20 @@ class KeyboardConfig(KeyboardConfigBase):
         return []
 
 
-    def do_get_keycode(self, client_keycode, keyname, pressed, modifiers, keystr, group):
+    def do_get_keycode(self, client_keycode:int, keyname:str, pressed:bool,
+                       modifiers, keyval:int, keystr:str, group:int) -> Tuple[int,int]:
         global KEYCODES
         keycode = KEYCODES.get(keyname, -1)
         if keycode==-1:
             keycode = KEYCODES.get(keyname.upper(), -1)
-        log("get_keycode%s=%s, %s", (client_keycode, keyname, pressed, modifiers, keystr, group), keycode, group)
+        log("get_keycode%s=%s, %s",
+            (client_keycode, keyname, pressed, modifiers, keyval, keystr, group), keycode, group)
         return keycode, group
 
 #we currently assume that all key events are sent using X11 names,
 #so we need to translate them to osx keys
 #http://x86osx.com/bbs/c_data/pds_comment/MacintoshToolboxEssentials.pdf
-KEYCODES = {
+KEYCODES : Dict[str,int] = {
             #Standardkeys
             "A"     : 0,
             "B"     : 11,
@@ -74,8 +75,6 @@ KEYCODES = {
             "0"     : 29,
             "minus" : 27,
             "plus"  : 69,       #KeypadPlus
-            "equal" : 24,
-            "equal" : 24,
             "bracketleft"   : 33,
             "bracketright"  : 30,
             "semicolon"     : 41,
@@ -103,6 +102,7 @@ KEYCODES = {
             "KP_9"  : 92,
             "KP_0"  : 82,
             "KP_Decimal"    : 65,
+            #"equal" : 24,
             "equal"         : 81,
             "KP_Divide"     : 75,
             "KP_Multiply"   : 67,
