@@ -1,10 +1,13 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from ctypes import cdll, WinDLL, c_void_p, Structure, cast, c_char, c_int, pointer, POINTER
+from ctypes import (
+	cdll, c_size_t, c_void_p, Structure, cast, c_char, c_int, pointer, POINTER,
+	WinDLL,  # @UnresolvedImport
+	)
 from ctypes.wintypes import HDC, HANDLE, BOOL, BYTE, LPCSTR, DWORD, WORD
 
 from xpra.platform.win32.common import CreateDCA, DeleteDC, gdi32
@@ -20,6 +23,12 @@ LPPRINTER_DEFAULTS = c_void_p
 PSECURITY_DESCRIPTOR = HANDLE
 
 msvcrt = cdll.msvcrt
+malloc = malloc
+malloc.argtypes = [c_size_t]
+malloc.restype = c_void_p
+free = free
+free.argtypes = [c_void_p]
+free.restype = None
 winspool = WinDLL('winspool.drv', use_last_error=True)
 OpenPrinterA = winspool.OpenPrinterA
 OpenPrinterA.restype = BOOL
@@ -41,17 +50,17 @@ class DOCINFO(Structure):
 		]
 
 LPDOCINFO = POINTER(DOCINFO)
-StartDocA = gdi32.StartDocA
+StartDocA = gdi32.StartDocA  # @UndefinedVariable
 StartDocA.argtypes = [HDC, LPDOCINFO]
 StartDocA.restype = int
-EndDoc = gdi32.EndDoc
+EndDoc = gdi32.EndDoc  # @UndefinedVariable
 EndDoc.argtypes = [HDC]
 EndDoc.restype = int
-StartPage = gdi32.StartPage
+StartPage = gdi32.StartPage  # @UndefinedVariable
 StartPage.argtypes = [HDC]
-EndPage = gdi32.EndPage
+EndPage = gdi32.EndPage  # @UndefinedVariable
 EndPage.argtypes = [HDC]
-TextOutA = gdi32.TextOutA
+TextOutA = gdi32.TextOutA  # @UndefinedVariable
 TextOutA.restype = BOOL
 TextOutA.argtypes = [HDC, c_int, c_int, LPCSTR, c_int]
 
@@ -109,7 +118,7 @@ class PRINTER_INFO_9(Structure):
 		]
 
 
-class GDIPrinterContext(object):
+class GDIPrinterContext:
 
 	def __init__(self, printer_name):
 		self.printer_name = printer_name
@@ -125,16 +134,16 @@ class GDIPrinterContext(object):
 		self.handle = HANDLE()
 		name = LPCSTR(self.printer_name)
 		if not OpenPrinterA(name, pointer(self.handle), None):
-			raise Exception("failed to open printer %s" % self.printer_name)
+			raise ValueError(f"failed to open printer {self.printer_name!r}")
 		log("OpenPrinter: handle=%#x", self.handle.value)
 		size = DWORD(0)
 		GetPrinterA(self.handle, 1, None, 0, pointer(size))
 		if size.value==0:
-			raise Exception("GetPrinterA PRINTER_INFO_1 failed for '%s'" % self.printer_name)
+			raise RuntimeError("GetPrinterA PRINTER_INFO_1 failed for '%s'" % self.printer_name)
 		log("GetPrinter: PRINTER_INFO_1 size=%#x", size.value)
-		self.info1 = msvcrt.malloc(size.value)
+		self.info1 = malloc(size.value)
 		if not GetPrinterA(self.handle, 1, self.info1, size.value, pointer(size)):
-			raise Exception("GetPrinterA PRINTER_INFO_1 failed for '%s'" % self.printer_name)
+			raise RuntimeError("GetPrinterA PRINTER_INFO_1 failed for '%s'" % self.printer_name)
 		info = cast(self.info1, POINTER(PRINTER_INFO_1))
 		log(" flags=%#x" % info[0].Flags)
 		log(" name=%#s" % info[0].pName)
@@ -144,18 +153,19 @@ class GDIPrinterContext(object):
 		size = DWORD(0)
 		GetPrinterA(self.handle, 2, None, 0, pointer(size))
 		if size.value==0:
-			raise Exception("GetPrinterA PRINTER_INFO_2 failed for '%s'" % self.printer_name)
+			raise RuntimeError("GetPrinterA PRINTER_INFO_2 failed for '%s'" % self.printer_name)
 		log("GetPrinter: PRINTER_INFO_2 size=%#x", size.value)
-		self.info2 = msvcrt.malloc(size.value)
+		self.info2 = malloc(size.value)
 		if GetPrinterA(self.handle, 2, self.info2, size.value, pointer(size)):
 			info = cast(self.info2, POINTER(PRINTER_INFO_2))
 			log(" driver=%#s" % info[0].pDriverName)
 
+		devmode = 0
 		size = DWORD(0)
 		GetPrinterA(self.handle, 8, None, 0, pointer(size))
 		if size.value==0:
-			raise Exception("GetPrinter: PRINTER_INFO_8 failed for '%s'" % self.printer_name)
-		self.info8 = msvcrt.malloc(size.value)
+			raise RuntimeError("GetPrinter: PRINTER_INFO_8 failed for '%s'" % self.printer_name)
+		self.info8 = malloc(size.value)
 		if GetPrinterA(self.handle, 8, self.info8, size.value, pointer(size)):
 			info = cast(self.info8, POINTER(PRINTER_INFO_8))
 			if info[0] and info[0].pDevMode:
@@ -166,16 +176,17 @@ class GDIPrinterContext(object):
 		size = DWORD(0)
 		GetPrinterA(self.handle, 9, None, 0, pointer(size))
 		if size.value==0:
-			raise Exception("GetPrinter: PRINTER_INFO_9 failed for '%s'" % self.printer_name)
+			raise RuntimeError("GetPrinter: PRINTER_INFO_9 failed for '%s'" % self.printer_name)
 		log("GetPrinter: PRINTER_INFO_9 size=%#x" % size.value)
-		self.info9 = msvcrt.malloc(size.value)
+		self.info9 = malloc(size.value)
 		if GetPrinterA(self.handle, 9, self.info9, size.value, pointer(size)):
 			info = cast(self.info9, POINTER(PRINTER_INFO_9))
 			if info[0] and info[0].pDevMode:
 				devmode = cast(info[0].pDevMode, POINTER(DEVMODE))
 				log("PRINTER_INFO_9: devmode=%s" % devmode)
 				log("PRINTER_INFO_9: device name=%s" % devmode[0].dmDeviceName)
-		assert devmode, "failed to query a DEVMODE for %s" % self.printer_name
+		if not devmode:
+			raise RuntimeError(f"failed to query a DEVMODE for {self.printer_name!r}")
 		self.hdc = CreateDCA(None, name, None, devmode)
 		log("CreateDCA(..)=%#x", self.hdc)
 		return self.hdc
@@ -186,16 +197,16 @@ class GDIPrinterContext(object):
 			DeleteDC(self.hdc)
 			self.hdc = None
 		if self.info1:
-			msvcrt.free(self.info1)
+			free(self.info1)
 			self.info1 = None
 		if self.info2:
-			msvcrt.free(self.info2)
+			free(self.info2)
 			self.info2 = None
 		if self.info8:
-			msvcrt.free(self.info8)
+			free(self.info8)
 			self.info8 = None
 		if self.info9:
-			msvcrt.free(self.info9)
+			free(self.info9)
 			self.info9 = None
 		if self.handle:
 			ClosePrinter(self.handle)
