@@ -33,7 +33,7 @@ from xpra.make_thread import start_thread
 from xpra.log import Logger
 
 log = Logger("proxy")
-authlog = Logger("proxy", "auth")
+authlog = Logger("auth")
 
 freeze_support()
 
@@ -674,7 +674,7 @@ class ProxyServer(ServerCore):
 
 
     def get_info(self, proto, *_args) -> Dict[str,Any]:
-        authenticated = proto and proto.authenticators
+        authenticated = bool(proto and proto.authenticators)
         if not authenticated:
             info = self.get_minimal_server_info()
         else:
